declare type AnyContract = Contract<any, AnyOptions>;

declare type AnyOptions = any;

declare type AnyProvider = Provider<any, AnyOptions>;

/**
 * Base class for typed errors, adding nested errors (causes) to the stack.
 *
 * @public
 */
export declare class BaseError extends Error {
    readonly message: string;
    readonly stack?: string;
    protected nested: Array<Error>;
    /**
     * Create a new typed error.
     *
     * @param message - error message
     * @param nested - internal errors (causes)
     */
    constructor(message: string, ...nested: Array<Error>);
    /**
     * Get the cause of this error: the first nested error.
     *
     * @public
     */
    get cause(): Error | undefined;
    /**
     * Get the number of nested errors.
     *
     * @public
     */
    get length(): number;
}

/**
 * Base interface for all constructor options.
 *
 * @public
 */
export declare interface BaseOptions {
    container: Container;
}

/**
 * Logger implementation using the console.
 *
 * Works in the browser or node.
 *
 * @public
 */
export declare class ConsoleLogger implements Logger {
    /**
     * Global instance for users needing no customization.
     */
    static readonly global: ConsoleLogger;
    /**
     * Create a child logger.
     */
    child(): ConsoleLogger;
    /**
     * Log a message at debug level.
     */
    debug(...params: Array<unknown>): void;
    /**
     * Log a message at info level.
     */
    info(...params: Array<unknown>): void;
    /**
     * Log a message at warning level.
     */
    warn(...params: Array<unknown>): void;
    /**
     * Log a message at error level.
     *
     * Writes to stdout in node, appears as a red message in chrome, etc.
     */
    error(...params: Array<unknown>): void;
}

/**
 * Some constructor taking options as the first parameter.
 *
 * @public
 */
export declare interface Constructor<TReturn, TOptions extends BaseOptions> {
    new (options: TOptions, ...extra: ExtraArgs): TReturn;
}

/**
 * Permanently attach a container to all instances of this class.
 *
 * @public
 */
export declare function constructWithContainer(container: Container): <TInner, TOptions extends BaseOptions>(target: Constructor<TInner, TOptions>) => WrappedConstructor<TInner, TOptions>;

/**
 * This is an exceptionally minimal DI container.
 *
 * @public
 */
export declare class Container implements ContainerOptions {
    static from(...modules: Array<Module>): Container;
    logger?: Logger;
    protected modules: Array<Module>;
    protected ready: boolean;
    constructor(modules: Iterable<Module>);
    /**
     * Configure each module, linking dependencies to their contracts and preparing factories.
     *
     * This must be done sequentially, since some modules may use classes from other modules.
     */
    configure(options?: ContainerOptions): Promise<this>;
    /**
     * Returns the best provided value for the request contract.
     */
    create<TReturn, TOptions extends BaseOptions>(contract: Contract<TReturn, TOptions>, options?: PartialOptions<TOptions>, ...args: ExtraArgs): Promise<TReturn>;
    debug(): void;
    getModules(): ReadonlyArray<Module>;
    /**
     * Create a child container with additional modules.
     */
    with(...modules: ReadonlyArray<Module>): Container;
    provide<TReturn, TOptions extends BaseOptions>(module: Module, contract: Contract<TReturn, TOptions>, options: PartialOptions<TOptions>, args: ExtraArgs): Promise<TReturn>;
    construct<TReturn, TOptions extends BaseOptions>(ctor: Constructor<TReturn, TOptions>, options: PartialOptions<TOptions>, args: ExtraArgs): Promise<any>;
    apply<TReturn, TOptions extends BaseOptions>(impl: Function, thisArg: unknown, options: PartialOptions<TOptions>, args: ExtraArgs): Promise<TReturn>;
    protected fail(msg: string): never;
    /**
     * Prepare a map with the dependencies for a descriptor.
     *
     * This will always inject the container itself to configure children.
     */
    protected dependencies<TOptions extends BaseOptions>(deps: Array<Dependency>, passed: PartialOptions<TOptions>): Promise<TOptions>;
}

/**
 * Error indicating that this container is already bound and cannot be bound again.
 *
 * @public
 */
export declare class ContainerBoundError extends BaseError {
    constructor(msg?: string, ...nested: Array<Error>);
}

/**
 * Error indicating that this container is not bound yet and not ready to be used.
 *
 * @public
 */
export declare class ContainerNotBoundError extends BaseError {
    constructor(msg?: string, ...nested: Array<Error>);
}

declare interface ContainerOptions {
    logger?: Logger;
}

/**
 * A contract identifier or concrete constructor.
 *
 * @public
 */
export declare type Contract<TReturn, TOptions extends BaseOptions> = ContractName | Constructor<TReturn, TOptions>;

/**
 * The identifier for a contract of some type.
 *
 * @public
 */
declare type ContractName = string | symbol;

/**
 * Get the standard name for a contract (usually a constructor).
 *
 * This accepts strings and symbols, so if a function is not provably a constructor, simply pass the name.
 *
 * @public
 */
export declare function contractName(c: Contract<unknown, AnyOptions>): ContractName;

declare interface Dependency {
    name: string | symbol;
    contract: AnyContract;
}

/**
 * Error indicating that the target descriptor could not be found on the decorator target.
 *
 * @public
 */
export declare class DescriptorNotFoundError extends BaseError {
    constructor(msg?: string, ...nested: Array<Error>);
}

/**
 * Extra arguments of unknown types.
 *
 * @public
 */
declare type ExtraArgs = ReadonlyArray<any>;

/**
 * Factory provider signature.
 *
 * @public
 */
declare type Factory<R, O extends BaseOptions> = (options: O) => Promise<R>;

/**
 * Fluent provider binding methods.
 *
 * @public
 */
declare interface FluentBinding<TContract, TReturn, TOptions extends BaseOptions> {
    toConstructor(implementation: Constructor<TContract, TOptions>): TReturn;
    toFactory(factory: Factory<TContract, TOptions>): TReturn;
    toInstance(instance: TContract): TReturn;
}

/**
 * Get attached dependencies.
 *
 * @param target - the previously-decorated target
 *
 * @public
 */
export declare function getInject(target: any): Array<Dependency>;

/**
 * Get dependencies provided by this previously-decorated target.
 *
 * @param target - the decorated target
 *
 * @public
 */
export declare function getProvides(target: any): Array<Dependency>;

/**
 * Injection decorator for classes.
 *
 * @param needs - dependencies required by the decorated target
 *
 * @public
 */
export declare function Inject(...needs: Array<InjectedDependency>): (target: any, key?: string, providedDesc?: PropertyDescriptor) => void;

declare type InjectedDependency = Dependency | AnyContract;

/**
 * Error indicating that the container was able to find a provider, but did not understand the provider type and could
 * not use it to fulfill the dependency.
 *
 * This should not normally occur, unless custom provider types are in use.
 *
 * @public
 */
export declare class InvalidProviderError extends BaseError {
    constructor(msg?: string, ...nested: Array<Error>);
}

/**
 * Error indicating that the decorator target is not valid.
 *
 * @public
 */
export declare class InvalidTargetError extends BaseError {
    constructor(msg?: string, ...nested: Array<Error>);
}

/**
 * The required signature for a function to be invoked by the wrapper.
 *
 * @public
 */
declare type InvokableFunction<TOptions, TReturn> = (options: TOptions, ...others: ExtraArgs) => TReturn;

/**
 * Permanently attach a container to all invocations of this function.
 *
 * @public
 */
export declare function invokeWithContainer<TReturn, TOptions extends BaseOptions>(container: Container, target: InvokableFunction<TOptions, TReturn>): InvokableFunction<WrappedOptions<TOptions>, TReturn>;

/**
 * Typeguard for constructors. Only really validates that `it` is a function.
 *
 * @public
 */
export declare function isConstructor(it: unknown): it is Constructor<unknown, AnyOptions>;

/**
 * Log interface to fit console or bunyan.
 *
 * @public
 */
export declare interface Logger {
    child(options: any): Logger;
    debug(msg: string, ...params: Array<unknown>): void;
    debug(options: LogParams, msg: string, ...params: Array<unknown>): void;
    error(msg: string, ...params: Array<unknown>): void;
    error(err: Error, msg: string, ...params: Array<unknown>): void;
    error(options: LogParams, msg: string, ...params: Array<unknown>): void;
    info(msg: string, ...params: Array<unknown>): void;
    info(options: LogParams, msg: string, ...params: Array<unknown>): void;
    warn(msg: string, ...params: Array<unknown>): void;
    warn(options: LogParams, msg: string, ...params: Array<unknown>): void;
}

/**
 * Error indicating that a debug method has been called, but no logger was registered for debugging.
 *
 * @public
 */
export declare class LoggerNotFoundError extends MissingValueError {
    constructor(msg?: string, ...nested: Array<Error>);
}

/**
 * Available log levels.
 *
 * @public
 */
export declare enum LogLevel {
    Debug = "debug",
    Error = "error",
    Info = "info",
    Warn = "warn",
    DEBUG = "debug",
    ERROR = "error",
    INFO = "info",
    WARN = "warn"
}

declare type LogParams = object;

/**
 * Switch helper to invoke log methods based on variable log level. Dispatches a call to the appropriate log level
 * method.
 *
 * @public
 */
export declare function logWithLevel(logger: Logger, level: LogLevel, options: Error | LogParams, msg: string): void;

/**
 * Error indicating that a required value is missing.
 *
 * @public
 */
export declare class MissingValueError extends BaseError {
    constructor(msg?: string, ...nested: Array<Error>);
}

/**
 * Provides a set of dependencies, bound in the `configure` method.
 *
 * @public
 */
export declare abstract class Module implements ModuleOptions {
    container?: Container;
    logger?: Logger;
    protected providers: Map<AnyContract, AnyProvider>;
    constructor();
    configure(options: ModuleOptions): Promise<void>;
    get<C, O extends BaseOptions>(contract: Contract<C, O>): Provider<C, O>;
    /**
     * Indicate if this module provides a dependency and if so, how.
     */
    has<C, O extends BaseOptions>(contract: Contract<C, O>): boolean;
    get size(): number;
    /**
     * Bind a provider to a contract. This is the core of the module.
     *
     * @param contract - the contract to be bound
     * @param type - the type of provider
     * @param value - the class, factory, or instance to bind
     */
    bindTo<C, I extends C, O extends BaseOptions>(contract: Contract<C, O>, type: ProviderType.Constructor, value: Constructor<I, O>): this;
    bindTo<C, I extends C, O extends BaseOptions>(contract: Contract<C, O>, type: ProviderType.Factory, value: Factory<I, O>): this;
    bindTo<C, I extends C, O extends BaseOptions>(contract: Contract<C, O>, type: ProviderType.Instance, value: I): this;
    bindTo<C, I extends C, O extends BaseOptions>(contract: Contract<C, O>, type: ProviderType.None): this;
    /**
     * Register a class as the provider for a particular contract. The class will be instantiated after having
     * dependencies resolved, its parameters being the dependencies and any additional arguments passed to the container.
     */
    bind<C, I extends C, O extends BaseOptions>(contract: Contract<C, O>): FluentBinding<I, this, O>;
    debug(): void;
    protected bindFunction<C, I extends C, O extends BaseOptions>(fn: Factory<I, O>): void;
    protected bindPrototype(proto: object): void;
}

/**
 * Required options for modules.
 *
 * @public
 */
export declare interface ModuleOptions {
    container?: Container;
    logger?: Logger;
}

/**
 * Logger implementation that consumes input and produces no output.
 *
 * @public
 */
export declare class NullLogger implements Logger {
    static readonly global: NullLogger;
    child(): NullLogger;
    debug(...data: Array<unknown>): void;
    info(...data: Array<unknown>): void;
    warn(...data: Array<unknown>): void;
    error(...data: Array<unknown>): void;
}

/**
 * A partial set of wrapped options.
 *
 * @public
 */
declare type PartialOptions<T extends BaseOptions> = Partial<WrappedOptions<T>>;

/**
 * Provider definitions.
 *
 * @public
 */
export declare type Provider<R, O extends BaseOptions> = {
    type: ProviderType.Constructor;
    value: Constructor<R, O>;
} | {
    type: ProviderType.Factory;
    value: Factory<R, O>;
} | {
    type: ProviderType.Instance;
    value: R;
} | {
    type: ProviderType.None;
    value: undefined;
};

/**
 * Providers for a particular contract.
 *
 * @public
 */
export declare enum ProviderType {
    None = 0,
    Constructor = 1,
    Factory = 2,
    Instance = 3
}

/**
 * Decorator for methods that can resolve or provide some dependency.
 *
 * @public
 */
export declare function Provides<TInjected>(...provides: Array<InjectedDependency>): (target: any, key: string, providedDesc?: PropertyDescriptor) => void;

declare interface WrappedConstructor<TInner, TOptions extends BaseOptions> extends Constructor<TInner, TOptions> {
    new (options: WrappedOptions<TOptions>): TInner;
}

/**
 * Options after being wrapped with a permanent container.
 *
 * @public
 */
declare type WrappedOptions<T extends BaseOptions> = Omit<T, 'container'>;

export { }
