import { Dependency } from './Dependency.js';
import { Logger } from './logger/Logger.js';
import { Module } from './Module.js';
/**
 * Extra arguments of unknown types.
 *
 * @public
 */
export declare type ExtraArgs = ReadonlyArray<any>;
/**
 * Some constructor taking options as the first parameter.
 *
 * @public
 */
export interface Constructor<TReturn, TOptions extends BaseOptions> {
    new (options: TOptions, ...extra: ExtraArgs): TReturn;
}
/**
 * The identifier for a contract of some type.
 *
 * @public
 */
export declare type ContractName = string | symbol;
/**
 * A contract identifier or concrete constructor.
 *
 * @public
 */
export declare type Contract<TReturn, TOptions extends BaseOptions> = ContractName | Constructor<TReturn, TOptions>;
export declare type AnyContract = Contract<any, AnyOptions>;
/**
 * Options after being wrapped with a permanent container.
 *
 * @public
 */
export declare type WrappedOptions<T extends BaseOptions> = Omit<T, 'container'>;
/**
 * A partial set of wrapped options.
 *
 * @public
 */
export declare type PartialOptions<T extends BaseOptions> = Partial<WrappedOptions<T>>;
export declare type AnyOptions = any;
/**
 * Get the standard name for a contract (usually a constructor).
 *
 * This accepts strings and symbols, so if a function is not provably a constructor, simply pass the name.
 *
 * @public
 */
export declare function contractName(c: Contract<unknown, AnyOptions>): ContractName;
/**
 * Typeguard for constructors. Only really validates that `it` is a function.
 *
 * @public
 */
export declare function isConstructor(it: unknown): it is Constructor<unknown, AnyOptions>;
/**
 * Base interface for all constructor options.
 *
 * @public
 */
export interface BaseOptions {
    container: Container;
}
export interface ContainerOptions {
    logger?: Logger;
}
/**
 * This is an exceptionally minimal DI container.
 *
 * @public
 */
export declare class Container implements ContainerOptions {
    static from(...modules: Array<Module>): Container;
    logger?: Logger;
    protected modules: Array<Module>;
    protected ready: boolean;
    constructor(modules: Iterable<Module>);
    /**
     * Configure each module, linking dependencies to their contracts and preparing factories.
     *
     * This must be done sequentially, since some modules may use classes from other modules.
     */
    configure(options?: ContainerOptions): Promise<this>;
    /**
     * Returns the best provided value for the request contract.
     */
    create<TReturn, TOptions extends BaseOptions>(contract: Contract<TReturn, TOptions>, options?: PartialOptions<TOptions>, ...args: ExtraArgs): Promise<TReturn>;
    debug(): void;
    getModules(): ReadonlyArray<Module>;
    /**
     * Create a child container with additional modules.
     */
    with(...modules: ReadonlyArray<Module>): Container;
    provide<TReturn, TOptions extends BaseOptions>(module: Module, contract: Contract<TReturn, TOptions>, options: PartialOptions<TOptions>, args: ExtraArgs): Promise<TReturn>;
    construct<TReturn, TOptions extends BaseOptions>(ctor: Constructor<TReturn, TOptions>, options: PartialOptions<TOptions>, args: ExtraArgs): Promise<any>;
    apply<TReturn, TOptions extends BaseOptions>(impl: Function, thisArg: unknown, options: PartialOptions<TOptions>, args: ExtraArgs): Promise<TReturn>;
    protected fail(msg: string): never;
    /**
     * Prepare a map with the dependencies for a descriptor.
     *
     * This will always inject the container itself to configure children.
     */
    protected dependencies<TOptions extends BaseOptions>(deps: Array<Dependency>, passed: PartialOptions<TOptions>): Promise<TOptions>;
}
export interface WrappedConstructor<TInner, TOptions extends BaseOptions> extends Constructor<TInner, TOptions> {
    new (options: WrappedOptions<TOptions>): TInner;
}
/**
 * Permanently attach a container to all instances of this class.
 *
 * @public
 */
export declare function constructWithContainer(container: Container): <TInner, TOptions extends BaseOptions>(target: Constructor<TInner, TOptions>) => WrappedConstructor<TInner, TOptions>;
/**
 * The required signature for a function to be invoked by the wrapper.
 *
 * @public
 */
export declare type InvokableFunction<TOptions, TReturn> = (options: TOptions, ...others: ExtraArgs) => TReturn;
export declare type WrappedFunction<TOptions extends BaseOptions, TReturn> = InvokableFunction<WrappedOptions<TOptions>, TReturn>;
/**
 * Permanently attach a container to all invocations of this function.
 *
 * @public
 */
export declare function invokeWithContainer<TReturn, TOptions extends BaseOptions>(container: Container, target: InvokableFunction<TOptions, TReturn>): InvokableFunction<WrappedOptions<TOptions>, TReturn>;
//# sourceMappingURL=Container.d.ts.map