import { ContainerBoundError } from './error/ContainerBoundError.js';
import { ContainerNotBoundError } from './error/ContainerNotBoundError.js';
import { InvalidProviderError } from './error/InvalidProviderError.js';
import { LoggerNotFoundError } from './error/LoggerNotFoundError.js';
import { MissingValueError } from './error/MissingValueError.js';
import { getInject } from './Inject.js';
import { ProviderType } from './Module.js';
import { isNil } from './utils/index.js';
import { VERSION_INFO } from './version.js';
/**
 * Get the standard name for a contract (usually a constructor).
 *
 * This accepts strings and symbols, so if a function is not provably a constructor, simply pass the name.
 *
 * @public
 */
export function contractName(c) {
    if (typeof c === 'function') {
        return c.name;
    }
    else {
        return c;
    }
}
/**
 * Typeguard for constructors. Only really validates that `it` is a function.
 *
 * @public
 */
export function isConstructor(it) {
    return typeof it === 'function';
}
/**
 * This is an exceptionally minimal DI container.
 *
 * @public
 */
export class Container {
    constructor(modules) {
        this.modules = Array.from(modules);
        this.ready = false;
    }
    static from(...modules) {
        return new Container(modules);
    }
    /**
     * Configure each module, linking dependencies to their contracts and preparing factories.
     *
     * This must be done sequentially, since some modules may use classes from other modules.
     */
    async configure(options = { logger: undefined }) {
        if (this.ready) {
            throw new ContainerBoundError('container already bound');
        }
        this.logger = options.logger;
        this.ready = true;
        for (const module of this.modules) {
            await module.configure({
                container: this,
                logger: this.logger,
            });
        }
        return this;
    }
    /**
     * Returns the best provided value for the request contract.
     */
    async create(contract, options = {}, ...args) {
        if (!this.ready) {
            throw new ContainerNotBoundError('container has not been configured yet');
        }
        if (isNil(contract)) {
            throw new MissingValueError('missing contract');
        }
        if (this.logger !== undefined) {
            this.logger.debug({ contract }, 'container create contract');
        }
        const module = this.modules.find((item) => item.has(contract));
        if (isNil(module)) {
            if (isConstructor(contract)) {
                return this.construct(contract, options, args);
            }
            throw new MissingValueError(`container has no provider for contract: ${contractName(contract).toString()}`);
        }
        if (this.logger !== undefined) {
            this.logger.debug({ module }, 'contract provided by module');
        }
        return this.provide(module, contract, options, args);
    }
    debug() {
        if (isNil(this.logger)) {
            throw new LoggerNotFoundError('container has no logger');
        }
        this.logger.debug({ version: VERSION_INFO }, 'container debug');
        for (const m of this.modules) {
            m.debug();
        }
    }
    getModules() {
        return this.modules;
    }
    /**
     * Create a child container with additional modules.
     */
    with(...modules) {
        const merged = this.modules.concat(modules);
        return new Container(merged);
    }
    async provide(module, contract, options, args) {
        const provider = module.get(contract);
        if (isNil(provider)) {
            this.fail(`module has no provider for contract: ${contractName(contract).toString()}`);
        }
        switch (provider.type) {
            case ProviderType.Constructor:
                return this.construct(provider.value, options, args);
            case ProviderType.Factory:
                return this.apply(provider.value, module, options, args);
            case ProviderType.Instance:
                return provider.value;
            default:
                throw new InvalidProviderError('invalid provider type');
        }
    }
    async construct(ctor, options, args) {
        const deps = [await this.dependencies(getInject(ctor), options)];
        return Reflect.construct(ctor, deps.concat(args));
    }
    async apply(
    // eslint-disable-next-line @typescript-eslint/ban-types
    impl, thisArg, options, args) {
        const deps = [await this.dependencies(getInject(impl), options)];
        return Reflect.apply(impl, thisArg, deps.concat(args));
    }
    fail(msg) {
        const error = new MissingValueError(msg);
        if (this.logger !== undefined) {
            this.logger.error(error, msg);
        }
        throw error;
    }
    /**
     * Prepare a map with the dependencies for a descriptor.
     *
     * This will always inject the container itself to configure children.
     */
    async dependencies(deps, passed) {
        const options = {};
        for (const dependency of deps) {
            const { contract, name } = dependency;
            if (!Reflect.has(passed, name)) {
                // tslint:disable-next-line:no-any
                const dep = await this.create(contract);
                options[name] = dep;
            }
        }
        Object.assign(options, passed, {
            container: this,
        });
        return options;
    }
}
/**
 * Permanently attach a container to all instances of this class.
 *
 * @public
 */
export function constructWithContainer(container) {
    return (target) => {
        // TODO: this shouldn't need any, but TInner is not sufficiently provable and causes a TS error
        class WrappedTarget extends target {
            constructor(options, ...others) {
                /* eslint-disable-next-line constructor-super */
                super(Object.assign(Object.assign({}, options), { container }), ...others);
            }
        }
        return WrappedTarget;
    };
}
/**
 * Permanently attach a container to all invocations of this function.
 *
 * @public
 */
export function invokeWithContainer(container, target) {
    /**
     * @this unknown
     */
    return function wrapper(options, ...others) {
        const completeOptions = Object.assign(Object.assign({}, options), { container });
        return target.apply(this, [completeOptions, ...others]);
    };
}
//# sourceMappingURL=Container.js.map