import { AnyContract, AnyOptions, BaseOptions, Constructor, Container, Contract } from './Container.js';
import { Logger } from './logger/Logger.js';
/**
 * Providers for a particular contract.
 *
 * @public
 */
export declare enum ProviderType {
    None = 0,
    Constructor = 1,
    Factory = 2,
    Instance = 3
}
/**
 * Factory provider signature.
 *
 * @public
 */
export declare type Factory<R, O extends BaseOptions> = (options: O) => Promise<R>;
/**
 * Concrete implementation provider signature group.
 *
 * @public
 */
export declare type Implementation<R, O extends BaseOptions> = Constructor<R, O> | Factory<R, O>;
/**
 * Provider definitions.
 *
 * @public
 */
export declare type Provider<R, O extends BaseOptions> = {
    type: ProviderType.Constructor;
    value: Constructor<R, O>;
} | {
    type: ProviderType.Factory;
    value: Factory<R, O>;
} | {
    type: ProviderType.Instance;
    value: R;
} | {
    type: ProviderType.None;
    value: undefined;
};
export declare type AnyProvider = Provider<any, AnyOptions>;
/**
 * Fluent provider binding methods.
 *
 * @public
 */
export interface FluentBinding<TContract, TReturn, TOptions extends BaseOptions> {
    toConstructor(implementation: Constructor<TContract, TOptions>): TReturn;
    toFactory(factory: Factory<TContract, TOptions>): TReturn;
    toInstance(instance: TContract): TReturn;
}
/**
 * Required options for modules.
 *
 * @public
 */
export interface ModuleOptions {
    container?: Container;
    logger?: Logger;
}
/**
 * Provides a set of dependencies, bound in the `configure` method.
 *
 * @public
 */
export declare abstract class Module implements ModuleOptions {
    container?: Container;
    logger?: Logger;
    protected providers: Map<AnyContract, AnyProvider>;
    constructor();
    configure(options: ModuleOptions): Promise<void>;
    get<C, O extends BaseOptions>(contract: Contract<C, O>): Provider<C, O>;
    /**
     * Indicate if this module provides a dependency and if so, how.
     */
    has<C, O extends BaseOptions>(contract: Contract<C, O>): boolean;
    get size(): number;
    /**
     * Bind a provider to a contract. This is the core of the module.
     *
     * @param contract - the contract to be bound
     * @param type - the type of provider
     * @param value - the class, factory, or instance to bind
     */
    bindTo<C, I extends C, O extends BaseOptions>(contract: Contract<C, O>, type: ProviderType.Constructor, value: Constructor<I, O>): this;
    bindTo<C, I extends C, O extends BaseOptions>(contract: Contract<C, O>, type: ProviderType.Factory, value: Factory<I, O>): this;
    bindTo<C, I extends C, O extends BaseOptions>(contract: Contract<C, O>, type: ProviderType.Instance, value: I): this;
    bindTo<C, I extends C, O extends BaseOptions>(contract: Contract<C, O>, type: ProviderType.None): this;
    /**
     * Register a class as the provider for a particular contract. The class will be instantiated after having
     * dependencies resolved, its parameters being the dependencies and any additional arguments passed to the container.
     */
    bind<C, I extends C, O extends BaseOptions>(contract: Contract<C, O>): FluentBinding<I, this, O>;
    debug(): void;
    protected bindFunction<C, I extends C, O extends BaseOptions>(fn: Factory<I, O>): void;
    protected bindPrototype(proto: object): void;
}
//# sourceMappingURL=Module.d.ts.map