import { contractName } from './Container.js';
import { LoggerNotFoundError } from './error/LoggerNotFoundError.js';
import { NullLogger } from './logger/NullLogger.js';
import { getProvides } from './Provides.js';
import { doesExist, isNil } from './utils/index.js';
/* eslint-disable @typescript-eslint/no-explicit-any */
/**
 * Providers for a particular contract.
 *
 * @public
 */
export var ProviderType;
(function (ProviderType) {
    ProviderType[ProviderType["None"] = 0] = "None";
    /* eslint-disable-next-line @typescript-eslint/no-shadow */
    ProviderType[ProviderType["Constructor"] = 1] = "Constructor";
    ProviderType[ProviderType["Factory"] = 2] = "Factory";
    ProviderType[ProviderType["Instance"] = 3] = "Instance";
})(ProviderType || (ProviderType = {}));
/**
 * Provides a set of dependencies, bound in the `configure` method.
 *
 * @public
 */
export class Module {
    constructor() {
        this.logger = NullLogger.global;
        this.providers = new Map();
    }
    async configure(options) {
        this.container = options.container;
        this.logger = options.logger;
        if (this.logger !== undefined) {
            this.logger.debug({ module: this }, 'configuring module');
        }
        const proto = Reflect.getPrototypeOf(this);
        if (proto) {
            this.bindPrototype(proto);
        }
    }
    get(contract) {
        const name = contractName(contract);
        const provider = this.providers.get(contract);
        if (this.logger !== undefined) {
            this.logger.debug({ contract: name, provider }, 'fetching contract from module');
        }
        return provider;
    }
    /**
     * Indicate if this module provides a dependency and if so, how.
     */
    has(contract) {
        const name = contractName(contract);
        if (this.logger !== undefined) {
            this.logger.debug({ contract: name }, 'searching module for contract');
        }
        return this.providers.has(contract);
    }
    get size() {
        return this.providers.size;
    }
    bindTo(contract, type, value) {
        const name = contractName(contract);
        if (this.logger !== undefined) {
            this.logger.debug({ contract: name, type, value }, 'binding contract');
        }
        this.providers.set(contract, { type, value });
        return this;
    }
    /**
     * Register a class as the provider for a particular contract. The class will be instantiated after having
     * dependencies resolved, its parameters being the dependencies and any additional arguments passed to the container.
     */
    bind(contract) {
        return {
            toConstructor: (constructor) => this.bindTo(contract, ProviderType.Constructor, constructor),
            toFactory: (factory) => this.bindTo(contract, ProviderType.Factory, factory),
            toInstance: (instance) => this.bindTo(contract, ProviderType.Instance, instance),
        };
    }
    debug() {
        if (isNil(this.logger)) {
            throw new LoggerNotFoundError('module has no logger');
        }
        this.logger.debug('module debug');
        for (const [module, provider] of this.providers.entries()) {
            this.logger.debug({ module, provider }, 'module provides contract');
        }
    }
    bindFunction(fn) {
        const provides = getProvides(fn);
        for (const p of provides) {
            this.bindTo(p.contract, ProviderType.Factory, fn);
        }
    }
    // eslint-disable-next-line @typescript-eslint/ban-types
    bindPrototype(proto) {
        for (const [/* name */ , desc] of Object.entries(Object.getOwnPropertyDescriptors(proto))) {
            if (typeof desc.value === 'function') {
                this.bindFunction(desc.value);
            }
        }
        const next = Reflect.getPrototypeOf(proto);
        if (doesExist(next) && next !== proto) {
            this.bindPrototype(next);
        }
    }
}
//# sourceMappingURL=Module.js.map