import { __decorate } from "tslib";
import { expect } from 'chai';
import { Container, DescriptorNotFoundError, InvalidTargetError, Module } from '../src/index.js';
import { getInject, Inject, injectionSymbol } from '../src/Inject.js';
/* eslint-disable @typescript-eslint/unbound-method */
describe('inject decorator', async () => {
    it('should be a decorator factory', async () => {
        expect(Inject()).to.be.a('function');
    });
    it('should work as a class decorator', async () => {
        const params = [{
                contract: 'a',
                name: 'a',
            }, {
                contract: 'b',
                name: 'b',
            }, {
                contract: 'c',
                name: 'c',
            }];
        let Target = class Target {
            /* c8 ignore next */
            method() { }
        };
        Target = __decorate([
            Inject(...params)
        ], Target);
        expect(getInject(Target)).to.deep.equal(params);
    });
    it('should attach dependencies', async () => {
        class FooClass {
        }
        const dep = {
            contract: FooClass,
            name: 'foo',
        };
        let TestClass = class TestClass {
        };
        TestClass = __decorate([
            Inject(dep)
        ], TestClass);
        expect(getInject(TestClass)).to.deep.equal([dep]);
    });
    it('should handle missing dependencies', async () => {
        class TestClass {
        }
        expect(getInject(TestClass)).to.deep.equal([]);
    });
    it('should flatten dependencies', async () => {
        class FooClass {
        }
        let TestClass = class TestClass {
        };
        TestClass = __decorate([
            Inject(FooClass)
        ], TestClass);
        expect(getInject(TestClass)).to.deep.equal([{
                contract: FooClass,
                name: FooClass.name,
            }]);
    });
    it('should look up the prototype chain', async () => {
        let FooClass = class FooClass {
        };
        FooClass = __decorate([
            Inject('foo')
        ], FooClass);
        let BarClass = class BarClass extends FooClass {
        };
        BarClass = __decorate([
            Inject('bar')
        ], BarClass);
        const EXPECTED_INJECTED = 2;
        const injected = getInject(BarClass);
        expect(injected.length).to.equal(EXPECTED_INJECTED);
        expect(injected[0].name, 'first injected option should be from parent class').to.equal('foo');
    });
    it('should fail on missing properties', async () => {
        class TestClass {
        }
        expect(() => {
            Inject()(TestClass, 'missing');
        }).to.throw();
    });
    it('should work when applied to methods', async () => {
        class TestClass {
            constructor() {
                this.foo = '';
            }
            bar(options) {
                this.foo = options.foo;
            }
        }
        __decorate([
            Inject('foo')
        ], TestClass.prototype, "bar", null);
        class TestModule extends Module {
            async configure(options) {
                await super.configure(options);
                this.bind('foo').toInstance('test');
            }
        }
        const ctr = Container.from(new TestModule());
        await ctr.configure();
        const foo = new TestClass();
        /* tslint:disable-next-line:no-unbound-method */
        await ctr.apply(foo.bar, foo, {}, []);
        expect(foo.foo).to.equal('test');
    });
    it('cannot be applied to non-function properties', async () => {
        expect(() => {
            /* c8 ignore next 4 */
            class TestField {
                constructor() {
                    this.foo = '';
                }
            }
            __decorate([
                Inject('foo')
            ], TestField.prototype, "foo", void 0);
            const foo = new TestField();
            expect(foo).to.equal(undefined);
        }).to.throw(DescriptorNotFoundError);
    });
    it('should throw when used on a property', async () => {
        const foo = {
            a: 1,
        };
        expect(() => {
            Inject('bar')(foo, 'a');
        }).to.throw(InvalidTargetError);
    });
    it('should return an empty array on undecorated classes', async () => {
        class Foo {
        }
        Reflect.set(Foo, injectionSymbol, {});
        expect(getInject(Foo)).to.deep.equal([]);
    });
});
//# sourceMappingURL=TestInject.js.map